# Create your views here.

from django.shortcuts import render_to_response

from mixpanel_django.backends import mp_backend 

def test_mp_view(request):
    
    mp_backend.trackMixpanelEvent('index page loaded', {'gender':'male'}, request)
    mp_backend.trackMixpanelEvent('index page loaded', {'gender':'female'}, request)
    
    mp_backend.trackfunnelMixpanelEvent('Signup', 1, 'opened front page', {'gender':'male'}, request)
    mp_backend.trackfunnelMixpanelEvent('Signup', 2, 'clicked signup button', {'gender':'male'}, request)
    mp_backend.trackfunnelMixpanelEvent('Signup', 3, 'confirmed email address ', {'gender':'male'}, request)
    return render_to_response('test_mp.html', {})


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from mixpanel_django.backends import dummy
import urllib2
import warnings

class DirectBackend(dummy.DummyBackend):
    
    def __init__(self, *args, **kwargs):
        super(DirectBackend, self).__init__(*args, **kwargs)
        handler = urllib2.HTTPHandler()
        self.connect = urllib2.build_opener(handler)
        self.connect.addheaders = [('User-agent', 'MixpanelDjango/0.1')]

    def send_event(self, url):
        '''
        overwrite me for other backends
        '''
        
        try:
            fp = self.connect.open(url)
        except:
            raise dummy.MixpanelError, 'send event error'
        res = fp.read()
        if res != '1':
            warnings.warn('event was not logged: %s' % url, dummy.MixpanelWarning)
        fp.close()

MixpanelBackend = DirectBackend


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from mixpanel_django.backends import dummy
import beanstalkc

class BeanstalkBackend(dummy.DummyBackend):

    def __init__(self, server, paramas):

        super(BeanstalkBackend, self).__init__(paramas)
        #self.connect = beanstalkc.Connection()
        self.host, self.port = server.split(':')
        self.paramas = paramas
        # set timeout value
        timeout = paramas.get('timeout', 30)
        import socket
        socket.setdefaulttimeout(float(timeout))
        
        # get beanstalk connection instance
        self.connect = self._do_connect()

    def _do_connect(self):
        # connect beanstalk server
        try:
            connect = beanstalkc.Connection(self.host, int(self.port), parse_yaml=False)
        except beanstalkc.SocketError:
            return None
        else:          
            # connected, then set beanstalk tube name, and watch name
            tube = self.paramas.get('tube', None)
            if tube:
                connect.use(tube)
                connect.watch(tube)
            else:
                connect.use('default')
                connect.watch('default')
            return connect
    
    def send_event(self, url):
        # check connection, did not got connection? try again
        if not self.connect:
            self.connect = self._do_connect()
            return
        
        # looks good,  put job
        try:
            self.connect.put(url)
        except beanstalkc.SocketError:
            # got error, set connect to None, until next time reconnect it
            self.connect = None
        
    def fetch_event(self):
        # check connection, did not got connection? try again
        if not self.connect:
            self.connect = self._do_connect()
            return
        
        # looks good,  reserve job
        try:
            return self.connect.reserve(0)
        except beanstalkc.SocketError:
            # got error, set connect to None, until next time reconnect it
            self.connect = None

MixpanelBackend = BeanstalkBackend



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from django.conf import settings
from django.utils import simplejson
from django.http import HttpRequest
import base64
import types
import time
import httpagentparser

class MixpanelError(Exception):pass
class MixpanelWarning(UserWarning):pass

class DummyBackend(object):

    def __init__(self, *args, **kwargs):

        self.base_properties = {}
        token = getattr(settings, 'MP_API_TOKEN', None)

        if not token:
            raise ValueError, 'Mixpanel API token not defined'
        self.base_properties['token'] = token
        
        # for other backend, you must has a connection instance
        self.connect = None

    def _build_mp_request_url(self, data):
        parma = base64.b64encode(simplejson.dumps(data))
        mp_request_url = "http://api.mixpanel.com/track/?data=" + parma
        return mp_request_url

    def fetch_event(self):
        raise NotImplementedError, '%s not implemented this method' % self.__class__.__name__
    
    def send_event(self, url):
        '''
        do send event, must overwrite this
        '''
        if not self.connect:
            pass

    def build_request_data(self, request):
        '''
        only get request ip address
        if you want to add more properties from request, you should overwrite this method
        @param request:
        '''
        if request.META.has_key('HTTP_X_REAL_IP'):
            ip = request.META['HTTP_X_REAL_IP']
        elif request.META.has_key('REMOTE_ADDR'):
            ip = request.META['REMOTE_ADDR']
        else:
            ip = '127.0.0.1'

        user_agent = request.META['HTTP_USER_AGENT']
        return {'ip': ip, 'user-agent':' '.join(httpagentparser.simple_detect(user_agent))}

    def trackMixpanelEvent(self, event_name, properties={}, request=None):
        '''
        do not overwrite this, unless you know what's you want.
        @param event_name:
        @param properties:
        @param request:
        '''
        #check input paramas
        if type(event_name) is not types.StringType:
            raise MixpanelError, 'event name must be a string'        
        if type(properties) is not types.DictType:
            raise MixpanelError, 'properties must be a dict'
        print isinstance(request, HttpRequest)
        if not isinstance(request, HttpRequest) and request != None:
            raise MixpanelError, 'request must is HttpRequest class instance or None value'

        # build rest data
        data = {}
        data['event'] = event_name
        properties = properties

        properties.update(self.base_properties)
        # append GMT time
        properties['time'] = int(time.mktime(time.gmtime(time.time())))
        
        if request:
            properties.update(self.build_request_data(request))

        data['properties'] = properties
        
        if settings.DEBUG:
            print data
            
        # send event
        self.send_event(self._build_mp_request_url(data))

    def trackfunnelMixpanelEvent(self, funnel, step, goal, properties={}, request=None):
        '''
        do not overwrite this, unless you know what's you want.
        @param funnel:
        @param step:
        @param goal:
        @param properties:
        @param request:
        '''
        #check input paramas 
        if type(funnel) is not types.StringType:
            raise MixpanelError, 'funnel must be a string'
        if type(step) is not types.IntType:
            raise MixpanelError, 'step must be a int number'
        if type(goal) is not types.StringType:
            raise MixpanelError, 'goal must be a string'            
        if type(properties) is not types.DictType:
            raise MixpanelError, 'properties must be a dict'
        if not isinstance(request, HttpRequest) and request != None:
            raise MixpanelError, 'request must is HttpRequest class instance or None value'
        
        # build data
        data = {}
        data['event'] = 'mp_funnel'
        
        properties = properties
        properties.update(self.base_properties)
        # append GMT time
        properties['time'] = int(time.mktime(time.gmtime(time.time())))
        properties['funnel'] = funnel
        properties['step'] = step
        properties['goal'] = goal
        
        if request:
            properties.update(self.build_request_data(request))
            
        data['properties'] = properties  
        
        if settings.DEBUG:
            print data        

        # send event
        self.send_event(self._build_mp_request_url(data))        
        
MixpanelBackend = DummyBackend







from django.conf import settings
from django.conf.urls.defaults import *

# Uncomment the next two lines to enable the admin:
# from django.contrib import admin
# admin.autodiscover()

urlpatterns = patterns('',
    # Example:
    (r'^$', include('mp_example.urls')),
    (r'^media/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.MEDIA_ROOT, 'show_indexes': True}),

)



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from django import template
from django.conf import settings
from django.template.loader import render_to_string
#from django.template import Node

register = template.Library()

MP_API_TOKEN = getattr(settings, 'MP_API_TOKEN', None)
if not MP_API_TOKEN:
    raise ValueError, 'Mixpanel API token not defined'
MP_EVENT_COMMON_JS_URL = getattr(settings, 'MP_EVENT_COMMON_JS_URL', '/media/scripts/mp_common_events.js')

@register.tag(name="mp_init")
def do_mp_init(parser, token):
    try:
        tag_name, format_string = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError, "%r tag requires a single argument" % token.contents.split()[0]
    if format_string not in ['standard', 'async', 'fbml']:
        raise template.TemplateSyntaxError, '%r tags argument should be one of "standard" or "async"' % tag_name
    return MPInitNode(format_string)

class MPInitNode(template.Node):
    def __init__(self, load_type):
        self.load_type = load_type
        self.context = {'mp_token':MP_API_TOKEN, 'mp_common_events_js':MP_EVENT_COMMON_JS_URL}
    def render(self, context):
        if self.load_type == 'standard':
            res = render_to_string('mp_init_standard.html', self.context)
        elif self.load_type == 'async':
            res = render_to_string('mp_init_async.html', self.context)
        elif self.load_type == 'fbml':
            res = render_to_string('mp_init_fbml.html', self.context)
        return res




#!/usr/bin/env python
#-*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name="mixpanel_django",
    version="0.1",
    package_dir={'mixpanel_django': 'mixpanel_django'},
    include_package_data=True,
    packages=find_packages(),
    package_data = {
        'mixpanel_django': [
            'media/scripts/*.js',
            'templates/*.html',
            'example/mp_example/templates/*.html'
        ],
    },
    install_requires=['django>=1.1', 'beanstalkc>=0.2', 'httpagentparser>=0.8.2'],

    # metadata for upload to PyPI
    author="fla.sam",
    author_email="fla.sam@gmail.com",
    description="django reuseable app, it can simply track events use mixpanel API",
    license="Apache Licence 2.0",
    keywords="python django mixpanel",
    url="https://bitbucket.org/fla.sam/mixpanel_django/", 
)



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from django.conf import settings
from django.core.cache import parse_backend_uri
from django.utils import importlib
from django.core import cache

BACKEND = {
    'dummy': 'dummy',
    'direct': 'direct',
    'beanstalk': 'beanstalk'
}

def _get_backend(backend_uri):
    scheme, host, params = parse_backend_uri(backend_uri)
    if scheme in BACKEND:
        name = 'mixpanel_django.backends.%s' % BACKEND[scheme]
    else:
        name = scheme
    module = importlib.import_module(name)
    return getattr(module, 'MixpanelBackend', None)(host, params)

mp_backend = _get_backend(getattr(settings, 'MP_BACKEND', 'dummy://'))





#!/usr/bin/env python
#-*- coding: utf-8 -*-

# We need some little fix to let consume running from any path
# you must set those two variables.
PROJECT_SETTINGS= 'mp_test.settings'
PROJECT_PARENT_PATH = '/home/sam/Aptana_Studio_Workspace/'
FETCH_JOB_INTERVAL = 5

# do NOT modify those code, because we need fix sys.path
import sys
import os
from django.utils import importlib # this module is very magic
from django.core.management import setup_environ
PROJECT_PATH = os.path.join(PROJECT_PARENT_PATH, PROJECT_SETTINGS.split('.')[0])
sys.path.insert(0, PROJECT_PARENT_PATH)
sys.path.insert(1, PROJECT_PATH)

# import project settings
settings = importlib.import_module(PROJECT_SETTINGS)
# put setting into runtime env
setup_environ(settings)

# now import mp_backend, more like normal django code... ha
from mixpanel_django.backends import mp_backend
import urllib2
import multiprocessing
import time

handler = urllib2.HTTPHandler()
opener = urllib2.build_opener(handler)
opener.addheaders = [('User-agent', 'MixpanelDjango/0.1')]
urllib2.install_opener(opener)

def push_event_worker(job):
    '''
    push event to Mixpanel
    @param job: job is string, that's we builded rest API request url
    '''
    try:
        fp = urllib2.urlopen(job)
        res = fp.read()
    except urllib2.URLError:
        return (False, job)
        
    if res == '1':
        return (True, job)
    else:
        # we accepted job not not logged on Mixpanel.
        return (True, job)
    
def main():
    # check system CPU numbers
    NUMBER_OF_PROCESS = multiprocessing.cpu_count() * 2
    
    # loop forever
    while 1:
        
        # preparing jobs list 
        jobs = []
        for x in range(NUMBER_OF_PROCESS):
            job = mp_backend.fetch_event()
            if job:
                jobs.append(job.body)
            else:
                continue
            # got job, immediately delete on beanstalk queue
            job.delete()
        
        # did not found any jobs, just continue next cycle
        if jobs == []:
            time.sleep(1)
            continue
        # ok jobs are ready, we starting multi processing pool
        pool = multiprocessing.Pool(NUMBER_OF_PROCESS)
        ress = pool.map(push_event_worker, jobs)
        
        # wait all jobs finished and terminat it
        pool.close()
        pool.join()
        pool.terminate()
        
        # check send event result
        for res, job in ress:
            # oops, got error, put job back
            if not res:
                mp_backend.send_event(job)
            
if __name__ == '__main__':
    main()